function [tau, F_CG, M_CG]= body_forces(F_rolling, F_air, Fx_wheel, Fy_wheel, Fz_wheel, delta, a_xyz, h_cg, m_car, l_front, l_rear, l_tread)
% calculates the vehicle body forces from rolling resistnace, drag and
% wheel forces

a_x = a_xyz(1);
a_y = a_xyz(2);

% wheel forces
Fx_fl = Fx_wheel(1);
Fx_fr = Fx_wheel(2);
Fx_rl = Fx_wheel(3);
Fx_rr = Fx_wheel(4);

Fy_fl = Fy_wheel(1);
Fy_fr = Fy_wheel(2);
Fy_rl = Fy_wheel(3);
Fy_rr = Fy_wheel(4);

Fz_fl = -Fz_wheel(1);
Fz_fr = -Fz_wheel(2);
Fz_rl = -Fz_wheel(3);
Fz_rr = -Fz_wheel(4);

% wheel positions
r_fl = [l_front  l_tread/2 -h_cg]';
r_fr = [l_front -l_tread/2 -h_cg]';
r_rl = [-l_rear  l_tread/2 -h_cg]';
r_rr = [-l_rear -l_tread/2 -h_cg]';

% rotation matrix from body frame to wheel frame
R = [cos(delta) -sin(delta) 0;
     sin(delta)  cos(delta) 0;
     0           0          1];
 
% front left wheel
F_fl = R*[Fx_fl Fy_fl Fz_fl]';
M_fl = cross(r_fl, F_fl);
 
% front right wheel
F_fr = R*[Fx_fr Fy_fr Fz_fr]';
M_fr = cross(r_fr, F_fr);

% rear left wheel
F_rl = [Fx_rl Fy_rl Fz_rl]';
M_rl = cross(r_rl, F_rl);

% rear right wheel
F_rr = [Fx_rr Fy_rr Fz_rr]';
M_rr = cross(r_rr, F_rr);

% sum of forces and moments about CG
F_CG = F_fl + F_fr + F_rl + F_rr - F_rolling - F_air;
M_CG = M_fl + M_fr + M_rl + M_rr + [-m_car*a_y*h_cg; m_car*a_x*h_cg; 0];
tau = [F_CG; M_CG];
