function [i, j, k, value] = calc_acc(input,delta0,omega_m0,V0,beta0,gain,l_front,l_rear,l_tread,m_car,r_wheel,road_condition,C_rolling,C_air,rho_air,A_front,A_side,...
        h_cg,I_car,r_fl,r_fr,theta_fl,theta_fr,l_car)
% finds vehicle acceleration for a set of inputs and states
% initialised variables

% gain is used if the search is being improved after initial value is found
input = input/gain;

% initiates the velocities
u0 = V0*cos(beta0);
v0 = V0*sin(beta0);
w0 = 0; 
linvel0 = [u0;v0;w0];


Vrot_fl = input*r_wheel; % rotational equivalent velocity for front left wheel

% yaw rate calculation
r0 = ((V0*(cos(beta0)*cos(delta0) + sin(beta0)*sin(delta0))...
    - Vrot_fl)/(r_fl*(sin(theta_fl)*cos(delta0)...
    - cos(theta_fl)*sin(delta0))));

% ground wheel contant point velocity calculation for front right wheel
v_wheel_fr0 = [V0*cos(beta0) + r0*r_fr*cos(theta_fr); V0*sin(beta0)...
    + r0*r_fr*sin(theta_fr)];
Vw_fr0 = norm(v_wheel_fr0);

% wheel angular speed calculation for front right wheel
omega_fr0 = (1/r_wheel)*(cos(delta0 -...
    atan2(v_wheel_fr0(2),v_wheel_fr0(1)))*Vw_fr0);

omega_wheel = [input;omega_fr0;omega_m0;omega_m0];
        
%simulate and find acceleration
[v_rotation0, v_wheel0, sigma_long0, sigma_lat0, sigma_res0, alpha0, beta]...
= wheel_slip(delta0, linvel0, r0, omega_wheel, l_front, l_rear, l_tread, r_wheel);
[my_long0, my_lat0] = friction_coefficient(linvel0, sigma_long0,...
    sigma_lat0, sigma_res0, road_condition);
Fz_wheel0 = vertical_wheel_forces(l_front, l_rear, m_car, l_car);
[Fx_wheel0, Fy_wheel0,X_R0]= horizontal_wheel_forces(Fz_wheel0,...
    my_long0, my_lat0, alpha0);
F_rolling0 = rolling_resistance(linvel0, C_rolling, m_car);
F_air0 = aerodynamic_drag(linvel0, C_air, rho_air, A_front, A_side);
[tau0, F_CG0, M_CG0]= body_forces(F_rolling0, F_air0, Fx_wheel0,...
Fy_wheel0, Fz_wheel0, delta0, zeros(3,1), h_cg, m_car, l_front,...
l_rear, l_tread);
nu_body0 = [u0;v0;w0;0;0;r0];
[nu_body_dot, a_body] = rigid_body_motion(nu_body0, tau0, m_car, I_car);

i = delta0;
j = omega_m0;
k = input;

% returns sum of the accelerations in body x, y and about z axis
value = abs(a_body(1)) + abs(a_body(2)) + abs(a_body(6));
