function [passed,deltabest,omega_mbest,omega_flbest,omega_frbest,totbestcheck,u_a0,i_a0,X_R0input,r0] = find_steadystate(V0,beta0,deltass,omega_mss,...
    omega_flss,valuess,stepsize,l_front,l_rear,l_tread,m_car,r_wheel,road_condition,C_rolling,C_air,rho_air,A_front,A_side,...
        h_cg,I_car,a1,a2,R_a,k_E,k_t,r_fl,r_fr,theta_fl,theta_fr,l_car,improve)
% use swipe on rear wheel angular velocity omega_m, front left wheel 
% angular velocity omega_fl and steering angle delta to find steady states

% initial best acceleration value
totbestcheck = 1e10;

n = 30;
if improve == 1
    % search is performed around the current best estimate if an intial
    deltabest = deltass;
    omega_mbest = omega_mss;
    omega_flbest = omega_flss;
    totbestcheck = valuess;
    gain = 100/stepsize;
else
    % gain set 1000 if new search is performed
    gain = 1000;
end

% makes a swipe in steering angle, rear wheel angular velocity and front
% left angular velocity
for k = 1:1:n
    if improve == 1
        % search around the best estimate
        delta0 = -(stepsize*k*pi/180) + deltass;
    else
        % search all negative steering angles between 0 and 30 deg
        delta0 = -(k*pi/180);
    end
   for j = 1:1:500
       if improve == 1
           % search around the best estimate
           omega_m0 = j*stepsize + omega_mss;
           lower = gain;
           upper = floor(omega_m0)*gain;
       else
           % search from scratch
           omega_m0 = j;
           lower = gain;
           upper = floor(omega_m0)*gain;
       end   
       % binary search to find best rear wheel angular velocity
       [delta0out, omega_m0out, omega_fl0out, value] = binary_search(lower, upper,delta0,omega_m0,V0,beta0,gain,l_front,l_rear,l_tread,m_car,r_wheel,road_condition,C_rolling,C_air,rho_air,A_front,A_side,...
        h_cg,I_car,r_fl,r_fr,theta_fl,theta_fr,l_car);
    
        % current best value update
        if value < totbestcheck
            deltabest = delta0out;
            omega_mbest = omega_m0out;
            omega_flbest = omega_fl0out;
            totbestcheck = value;
       end
   end
end
        
% computes the throttle value
u0 = V0*cos(beta0);
v0 = V0*sin(beta0);
w0 = 0; 
linvel0 = [u0;v0;w0];

Vrot_fl = omega_flbest*r_wheel;
r0 = ((V0*(cos(beta0)*cos(deltabest) + sin(beta0)*sin(deltabest)) - Vrot_fl)/(r_fl*(sin(theta_fl)*cos(deltabest) - cos(theta_fl)*sin(deltabest))));
v_wheel_fr0 = [V0*cos(beta0) + r0*r_fr*cos(theta_fr); V0*sin(beta0) + r0*r_fr*sin(theta_fr)];
Vw_fr0 = norm(v_wheel_fr0);
omega_fr0 = (1/r_wheel)*(cos(deltabest - atan2(v_wheel_fr0(2),v_wheel_fr0(1)))*Vw_fr0);
omega_wheel = [omega_flbest;omega_fr0;omega_mbest;omega_mbest];

[v_rotation0, v_wheel0, sigma_long0, sigma_lat0, sigma_res0, alpha0, beta] = wheel_slip(deltabest, linvel0, r0, omega_wheel, l_front, l_rear, l_tread, r_wheel);
[my_long0, my_lat0] = friction_coefficient(linvel0, sigma_long0, sigma_lat0, sigma_res0, road_condition);
Fz_wheel0 = vertical_wheel_forces(l_front, l_rear, m_car, l_car);
[Fx_wheel0, Fy_wheel0,X_R0]= horizontal_wheel_forces(Fz_wheel0, my_long0, my_lat0, alpha0);
F_rolling0 = rolling_resistance(linvel0, C_rolling, m_car);
F_air0 = aerodynamic_drag(linvel0, C_air, rho_air, A_front, A_side);
[tau0, F_CG0, M_CG0]= body_forces(F_rolling0, F_air0, Fx_wheel0, Fy_wheel0, Fz_wheel0, deltabest, zeros(3,1), h_cg, m_car, l_front, l_rear, l_tread);
nu_body0 = [u0;v0;w0;0;0;r0];
[nu_body_dot, a_body] = rigid_body_motion(nu_body0, tau0, m_car, I_car);
        
omega_frbest = omega_fr0;        
i_a0 = r_wheel*X_R0/k_t; 
u_a0 = k_E*omega_mbest + R_a*i_a0; % throttle value at steady state

% check if contraint is met
if u_a0 <= 25 && u_a0 >= 0
    passed = 1;
else
    passed = 0;
end

% controller force input values at steady state
Y0input = (1+l_front^2)*cos(delta0)*a2*(delta0*180/pi - beta0 - ((l_front*r0)/V0));
X_R0input = a1*u_a0 - ((Y0input*tan(delta0))/(1 + l_front^2));
