function [my_long, my_lat]= friction_coefficient(v_body, sigma_long, sigma_lat, sigma_res, road_condition)

% vehicle speed in body frame
V = norm([v_body(1) v_body(2)]);

% slip ratio in longitudinal and lateral direction
ratio_long_fl = sigma_long(1) /  sigma_res(1);
ratio_long_fr = sigma_long(2) /  sigma_res(2);
ratio_long_rl = sigma_long(3) /  sigma_res(3);
ratio_long_rr = sigma_long(4) /  sigma_res(4);
ratio_lat_fl = sigma_lat(1) /  sigma_res(1);
ratio_lat_fr = sigma_lat(2) /  sigma_res(2);
ratio_lat_rl = sigma_lat(3) /  sigma_res(3);
ratio_lat_rr = sigma_lat(4) /  sigma_res(4);

if isnan(ratio_long_fl); ratio_long_fl = 0; end
if isnan(ratio_long_fr); ratio_long_fr = 0; end
if isnan(ratio_long_rl); ratio_long_rl = 0; end
if isnan(ratio_long_rr); ratio_long_rr = 0; end
if isnan(ratio_lat_fl); ratio_lat_fl = 0; end
if isnan(ratio_lat_fr); ratio_lat_fr = 0; end
if isnan(ratio_lat_rl); ratio_lat_rl = 0; end
if isnan(ratio_lat_rr); ratio_lat_rr = 0; end

switch road_condition
    case 1
        % dry asphalt
        c1 = 1.2801;
        c2 = 23.99;
        c3 = 0.52;
        c4 = 0.03;

    case 2
        % wet asphalt
        c1 = 0.857;
        c2 = 33.822;
        c3 = 0.347;
        c4 = 0.03;

    case 3
        % dry concrete
        c1 = 1.1973;
        c2 = 25.168;
        c3 = 0.5373;
        c4 = 0.03;

    case 4
        % dry cobblestones
        c1 = 1.3713;
        c2 = 6.4565;
        c3 = 0.6691;
        c4 = 0.03;

    case 5
        % wet cobblestones
        c1 = 0.4004;
        c2 = 33.708;
        c3 = 0.1204;
        c4 = 0.03;

    case 6
        % snow
        c1 = 0.1946;
        c2 = 94.129;
        c3 = 0.0646;
        c4 = 0.03;

    case 7
        % ice
        c1 = 0.05;
        c2 = 306.39;
        c3 = 0;
        c4 = 0.03;
            
    otherwise
        c1 = 0;
        c2 = 0;
        c3 = 0;
        c4 = 0;

end

% resultant friction coefficient
my_fl = (c1*(1 - exp(-c2*sigma_res(1))) - c3*sigma_res(1))*exp(-c4*sigma_res(1)*V);
my_fr = (c1*(1 - exp(-c2*sigma_res(2))) - c3*sigma_res(2))*exp(-c4*sigma_res(2)*V);
my_rl = (c1*(1 - exp(-c2*sigma_res(3))) - c3*sigma_res(3))*exp(-c4*sigma_res(3)*V);
my_rr = (c1*(1 - exp(-c2*sigma_res(4))) - c3*sigma_res(4))*exp(-c4*sigma_res(4)*V);

% friction coefficient in longitudinal direction
my_long = [my_fl*ratio_long_fl
           my_fr*ratio_long_fr
           my_rl*ratio_long_rl
           my_rr*ratio_long_rr];

% friction coefficient in lateral direction
my_lat = [my_fl*ratio_lat_fl
          my_fr*ratio_lat_fr
          my_rl*ratio_lat_rl
          my_rr*ratio_lat_rr];

