load('steady_state_sim')
%% Simulation settings
clc;
close all
simtime = 20;                  % simulation time
noise = 0;                      % enable noise on IMU measurements, 0=off, 1=on
sampletime = -1;                % IMU sample time, -1=inherited
diff_lock = 1;                  % differential lock, 0=off, 1=on
wheel_drive = 2;                % choose between  1=front wheel drive, 2=rear wheel drive, 3=all wheel drive
road_condition = 5;             % road condition, 1=dry asphalt, 2=wet asphalt, 3=dry concrete
                                %                4=dry cobblestones, 5=wet cobblestones 6=snow, 7=ice

% initial conditions
V0 = 3;                         % initial speed [m/s]
pos0    = [0 0 0];              % initial position (north, west, up) [m]
psi0    = -0.1974;              % initial heading [rad]
linvel0 = [V0*cos(beta0) V0*sin(beta0) 0];           % initial translatory velocity (u,v,w) [m/s]
betamax = 10;                   % limit for desired sideslip angle from guidance [deg]

% dc motor constants
R_a = 0.8;                      % armature resistance [ohm]
L_a = 0.001;                    % armature inductance [H]
k_E = 0.17;                     % back EMF constant [V*s/rad]
k_t = 0.17;                     % torque constant [Nm/A]
I_motor = 0.002;                % dc motor moment of inertia [kg*m^2]

% vehicle constants
l_front = 0.30;                 % length from CG to front axle [m]
l_rear  = 0.26;                 % length from CG to rear axle [m]
l_tread = 0.38;                 % wheel tread [m]
h_cg = 0.075;                   % height of centre of gravity [m]
A_front = 0.0285;               % vehicle front and side area [m^2]
A_side = 0.0420;                % vehicle front and side area [m^2]
m_car = 7.0;                    % vehicle mass [kg]
I_z = 0.4535*1.3;                   % moment of inertia around vehicle z-axis [kg*m^2]
I_wheel = 0.0012;               % wheel moment of inertia [kg*m^2]  (Tarmac Buster)
m_wheel = 0.400;                % wheel mass [kg] (Tarmac Buster)
r_wheel = 0.085;                % wheel radius [m] (Tarmac Buster)
w_wheel = 0.060;                % wheel width [m] (Tarmac Buster)
C_air = 1.07;                   % air resistance coefficient
C_rolling = 0.01;               % rolling resistance coefficient

l_car = l_front + l_rear;                   % length of car [m]
I_x = (m_car/12)*(h_cg^2 + l_tread^2);      % moment of inertia around vehicle x-axis [kg*m^2]
I_y = (m_car/12)*(l_car^2 + h_cg^2);        % moment of inertia around vehicle y-axis [kg*m^2]
I_car = [I_x I_y I_z];                      % vector of vehicle moments of inertia
rho_air = 1.2928;                           % air density (kg/m^3)
mu = 0.4;                                   % friction coefficient for wet cobblestones

% path planning (North, East, Speed, path-fixed rotation}
wp_speed = V0;
waypoints = [[0,0,wp_speed,0],[30,-6,wp_speed,0],[30,0,wp_speed,0],[0,-6,wp_speed,0],[0,0,wp_speed,0]];

% finding the path-fixed rotations
for k = 8:4:length(waypoints)
    if (waypoints(k-2)-waypoints(k-6)) < 0 && (waypoints(k-3)-waypoints(k-7)) < 0
        waypoints(k) = atan((waypoints(k-2)-waypoints(k-6))/(waypoints(k-3)-waypoints(k-7)))+pi;
    else
        waypoints(k) = atan2((waypoints(k-2)-waypoints(k-6)),(waypoints(k-3)-waypoints(k-7)));
    end
end

sim carsim;
plot_figures;
hold off
disp('simulation has ended successfully')
