function [A,B] = linearize(V0,beta0,psidot0,delta0,omega_m0,omega_fl0,dmylong_du_fl,dmylong_du_fr,dmylong_du_rl,dmylong_du_rr, dmylong_dv_fl,dmylong_dv_fr,dmylong_dv_rl,dmylong_dv_rr,...
    dmylong_dpsidot_fl,dmylong_dpsidot_fr,dmylong_dpsidot_rl,dmylong_dpsidot_rr,dmylat_du_fl,dmylat_du_fr,dmylat_du_rl,dmylat_du_rr,...
    dmylat_dv_fl,dmylat_dv_fr,dmylat_dv_rl,dmylat_dv_rr, dmylat_dpsidot_fl,dmylat_dpsidot_fr,dmylat_dpsidot_rl,dmylat_dpsidot_rr,...
    dmylong_ddelta_fl,dmylong_ddelta_fr,dmylong_ddelta_rl,dmylong_ddelta_rr,dmylat_ddelta_fl,dmylat_ddelta_fr,dmylat_ddelta_rl,dmylat_ddelta_rr,...
    dmylong_dXR_fl,dmylong_dXR_fr,dmylong_dXR_rl,dmylong_dXR_rr,...
    dmylat_dXR_fl,dmylat_dXR_fr,dmylat_dXR_rl,dmylat_dXR_rr,mylong,mylat,l_front,l_rear,l_tread,l_car,A_front,A_side,m_car,I_z,C_air,C_rolling,rho_air,g,...
    r_fl,r_fr,r_rl,r_rr,theta_fl,theta_fr,theta_rl,theta_rr)

% linear velocities
u0 = V0*cos(beta0);
v0 = V0*sin(beta0);

% longitudinal friction coefficient
mylong_fl = mylong(1);
mylong_fr = mylong(2);
mylong_rl = mylong(3);
mylong_rr = mylong(4);

% longitudinal friction coefficient
mylat_fl = mylat(1);
mylat_fr = mylat(2);
mylat_rl = mylat(3);
mylat_rr = mylat(4);

% wheel center velocities
v_wheel_fl = [u0 - psidot0*r_fl*sin(theta_fl); v0 + psidot0*r_fl*cos(theta_fl)];
v_wheel_fr = [u0 + psidot0*r_fr*cos(theta_fr); v0 + psidot0*r_fr*sin(theta_fr)];
v_wheel_rl = [u0 - psidot0*r_rl*cos(theta_rl); v0 - psidot0*r_rl*sin(theta_rl)];
v_wheel_rr = [u0 + psidot0*r_rr*sin(theta_rr); v0 - psidot0*r_rr*cos(theta_rr)];

% wheel slip angles
alpha_fl = delta0 - atan2(v_wheel_fl(2) , v_wheel_fl(1));
alpha_fr = delta0 - atan2(v_wheel_fr(2) , v_wheel_fr(1));
alpha_rl = -atan2(v_wheel_rl(2) , v_wheel_rl(1));
alpha_rr = -atan2(v_wheel_rr(2) , v_wheel_rr(1));

% alpha derivatives
dalpha_du_fl = v_wheel_fl(2)/(v_wheel_fl(1)^2 + v_wheel_fl(2)^2);
dalpha_dv_fl = -v_wheel_fl(1)/(v_wheel_fl(1)^2 + v_wheel_fl(2)^2);
dalpha_dpsidot_fl = -r_fl*(u0*cos(theta_fl)+ v0*sin(theta_fl))/(v_wheel_fl(1)^2 + v_wheel_fl(2)^2);
dalpha_du_fr = v_wheel_fr(2)/(v_wheel_fr(1)^2 + v_wheel_fr(2)^2);
dalpha_dv_fr = -v_wheel_fr(1)/(v_wheel_fr(1)^2 + v_wheel_fr(2)^2);
dalpha_dpsidot_fr = r_fr*(v0*cos(theta_fr) - u0*sin(theta_fr))/(v_wheel_fr(1)^2 + v_wheel_fr(2)^2);
dalpha_du_rl = v_wheel_rl(2)/(v_wheel_rl(1)^2 + v_wheel_rl(2)^2);
dalpha_dv_rl = -v_wheel_rl(1)/(v_wheel_rl(1)^2 + v_wheel_rl(2)^2);
dalpha_dpsidot_rl = r_rl*(u0*sin(theta_rl) - v0*cos(theta_rl))/(v_wheel_rl(1)^2 + v_wheel_rl(2)^2);
dalpha_du_rr = v_wheel_rr(2)/(v_wheel_rr(1)^2 + v_wheel_rr(2)^2);
dalpha_dv_rr = -v_wheel_rr(1)/(v_wheel_rr(1)^2 + v_wheel_rr(2)^2);
dalpha_dpsidot_rr = r_rr*(u0*cos(theta_rr) + v0*sin(theta_rr))/(v_wheel_rr(1)^2 + v_wheel_rr(2)^2);

% normal forces
Zb_fl = -0.5*m_car*(l_rear*g/l_car);
Zb_fr = -0.5*m_car*(l_rear*g/l_car);
Zb_rl = -0.5*m_car*(l_front*g/l_car);
Zb_rr = -0.5*m_car*(l_front*g/l_car);

% wheel forces
X_fl = Zb_fl*(mylong_fl*cos(alpha_fl) + mylat_fl*sin(alpha_fl));
X_fr = Zb_fr*(mylong_fr*cos(alpha_fr) + mylat_fr*sin(alpha_fr));
Y_fl = Zb_fl*(-mylong_fl*sin(alpha_fl) + mylat_fl*cos(alpha_fl));
Y_fr = Zb_fr*(-mylong_fr*sin(alpha_fr) + mylat_fr*cos(alpha_fr));

% wheel force derivatives
dX_du_fl = Zb_fl*(dmylong_du_fl*cos(alpha_fl) + dmylat_du_fl*sin(alpha_fl)...
    + (mylat_fl*cos(alpha_fl) - mylong_fl*sin(alpha_fl))*dalpha_du_fl);
dX_dv_fl = Zb_fl*(dmylong_dv_fl*cos(alpha_fl) + dmylat_dv_fl*sin(alpha_fl)...
    + (mylat_fl*cos(alpha_fl) - mylong_fl*sin(alpha_fl))*dalpha_dv_fl);
dX_dpsidot_fl = Zb_fl*(dmylong_dpsidot_fl*cos(alpha_fl) + dmylat_dpsidot_fl*sin(alpha_fl)...
    + (mylat_fl*cos(alpha_fl) - mylong_fl*sin(alpha_fl))*dalpha_dpsidot_fl);
dX_ddelta_fl = Zb_fl*(dmylong_ddelta_fl*cos(alpha_fl) + dmylat_ddelta_fl*sin(alpha_fl)...
        + (mylat_fl*cos(alpha_fl) - mylong_fl*sin(alpha_fl)));
dX_dXR_fl = Zb_fl*(dmylong_dXR_fl*cos(alpha_fl) + dmylat_dXR_fl*sin(alpha_fl));

dX_du_fr = Zb_fr*(dmylong_du_fr*cos(alpha_fr) + dmylat_du_fr*sin(alpha_fr)...
    + (mylat_fr*cos(alpha_fr) - mylong_fr*sin(alpha_fr))*dalpha_du_fr);
dX_dv_fr = Zb_fr*(dmylong_dv_fr*cos(alpha_fr) + dmylat_dv_fr*sin(alpha_fr)...
    + (mylat_fr*cos(alpha_fr) - mylong_fr*sin(alpha_fr))*dalpha_dv_fr);
dX_dpsidot_fr = Zb_fr*(dmylong_dpsidot_fr*cos(alpha_fr) + dmylat_dpsidot_fr*sin(alpha_fr)...
    + (mylat_fr*cos(alpha_fr) - mylong_fr*sin(alpha_fr))*dalpha_dpsidot_fr);
dX_ddelta_fr = Zb_fr*(dmylong_ddelta_fr*cos(alpha_fr) + dmylat_ddelta_fr*sin(alpha_fr)...
        + (mylat_fr*cos(alpha_fr) - mylong_fr*sin(alpha_fr)));
dX_dXR_fr = Zb_fr*(dmylong_dXR_fr*cos(alpha_fr) + dmylat_dXR_fr*sin(alpha_fr));

dX_du_rl = 0;
dX_dv_rl = 0;
dX_dpsidot_rl = 0;
dX_ddelta_rl = 0;
dX_dXR_rl = 1/2;

dX_du_rr = 0;
dX_dv_rr = 0;
dX_dpsidot_rr = 0;
dX_ddelta_rr = 0;
dX_dXR_rr = 1/2;

dY_du_fl = Zb_fl*(dmylat_du_fl*cos(alpha_fl) - dmylong_du_fl*sin(alpha_fl)...
    - (mylong_fl*cos(alpha_fl) + mylat_fl*sin(alpha_fl))*dalpha_du_fl);
dY_dv_fl = Zb_fl*(dmylat_dv_fl*cos(alpha_fl) - dmylong_dv_fl*sin(alpha_fl)...
    - (mylong_fl*cos(alpha_fl) + mylat_fl*sin(alpha_fl))*dalpha_dv_fl);
dY_dpsidot_fl = Zb_fl*(dmylat_dpsidot_fl*cos(alpha_fl) - dmylong_dpsidot_fl*sin(alpha_fl)...
    - (mylong_fl*cos(alpha_fl) + mylat_fl*sin(alpha_fl))*dalpha_dpsidot_fl);
dY_ddelta_fl = Zb_fl*(dmylat_ddelta_fl*cos(alpha_fl) - dmylong_ddelta_fl*sin(alpha_fl)...
    - (mylong_fl*cos(alpha_fl) + mylat_fl*sin(alpha_fl)));
dY_dXR_fl = Zb_fl*(dmylat_dXR_fl*cos(alpha_fl) - dmylong_dXR_fl*sin(alpha_fl));

dY_du_fr = Zb_fr*(dmylat_du_fr*cos(alpha_fr) - dmylong_du_fr*sin(alpha_fr)...
    - (mylong_fr*cos(alpha_fr) + mylat_fr*sin(alpha_fr))*dalpha_du_fr);
dY_dv_fr = Zb_fr*(dmylat_dv_fr*cos(alpha_fr) - dmylong_dv_fr*sin(alpha_fr)...
    - (mylong_fr*cos(alpha_fr) + mylat_fr*sin(alpha_fr))*dalpha_dv_fr);
dY_dpsidot_fr = Zb_fr*(dmylat_dpsidot_fr*cos(alpha_fr) - dmylong_dpsidot_fr*sin(alpha_fr)...
    - (mylong_fr*cos(alpha_fr) + mylat_fr*sin(alpha_fr))*dalpha_dpsidot_fr);
dY_ddelta_fr = Zb_fr*(dmylat_ddelta_fr*cos(alpha_fr) - dmylong_ddelta_fr*sin(alpha_fr)...
    - (mylong_fr*cos(alpha_fr) + mylat_fr*sin(alpha_fr)));
dY_dXR_fr = Zb_fr*(dmylat_dXR_fr*cos(alpha_fr) - dmylong_dXR_fr*sin(alpha_fr));

dY_du_rl = Zb_rl*(dmylat_du_rl*cos(alpha_rl) - dmylong_du_rl*sin(alpha_rl)...
    - (mylong_rl*cos(alpha_rl) + mylat_rl*sin(alpha_rl))*dalpha_du_rl);
dY_dv_rl = Zb_rl*(dmylat_dv_rl*cos(alpha_rl) - dmylong_dv_rl*sin(alpha_rl)...
    - (mylong_rl*cos(alpha_rl) + mylat_rl*sin(alpha_rl))*dalpha_dv_rl);
dY_dpsidot_rl = Zb_rl*(dmylat_dpsidot_rl*cos(alpha_rl) - dmylong_dpsidot_rl*sin(alpha_rl)...
    - (mylong_rl*cos(alpha_rl) + mylat_rl*sin(alpha_rl))*dalpha_dpsidot_rl);
dY_ddelta_rl = Zb_rl*(dmylat_ddelta_rl*cos(alpha_rl) - dmylong_ddelta_rl*sin(alpha_rl)...
    - (mylong_rl*cos(alpha_rl) + mylat_rl*sin(alpha_rl)));
dY_dXR_rl = Zb_rl*(dmylat_dXR_rl*cos(alpha_rl) - dmylong_dXR_rl*sin(alpha_rl));

dY_du_rr = Zb_rr*(dmylat_du_rr*cos(alpha_rr) - dmylong_du_rr*sin(alpha_rr)...
    - (mylong_rr*cos(alpha_rr) + mylat_rr*sin(alpha_rr))*dalpha_du_rr);
dY_dv_rr = Zb_rr*(dmylat_dv_rr*cos(alpha_rr) - dmylong_dv_rr*sin(alpha_rr)...
    - (mylong_rr*cos(alpha_rr) + mylat_rr*sin(alpha_rr))*dalpha_dv_rr);
dY_dpsidot_rr = Zb_rr*(dmylat_dpsidot_rr*cos(alpha_rr) - dmylong_dpsidot_rr*sin(alpha_rr)...
    - (mylong_rr*cos(alpha_rr) + mylat_rr*sin(alpha_rr))*dalpha_dpsidot_rr);
dY_ddelta_rr = Zb_rr*(dmylat_ddelta_rr*cos(alpha_rr) - dmylong_ddelta_rr*sin(alpha_rr)...
    - (mylong_rr*cos(alpha_rr) + mylat_rr*sin(alpha_rr)));
dY_dXR_rr = Zb_rr*(dmylat_dXR_rr*cos(alpha_rr) - dmylong_dXR_rr*sin(alpha_rr));

% A matrix elements
A11 = (1/m_car)*((dX_du_fl + dX_du_fr)*cos(delta0) - (dY_du_fl + dY_du_fr)*sin(delta0)...
    + dX_du_rl + dX_du_rr) - C_rolling*g - rho_air*abs(u0)*A_front*C_air/m_car;

A12 = (1/m_car)*((dX_dv_fl + dX_dv_fr)*cos(delta0) - (dY_dv_fl + dY_dv_fr)*sin(delta0)...
    + dX_dv_rl + dX_dv_rr) + psidot0;

A13 = (1/m_car)*((dX_dpsidot_fl + dX_dpsidot_fr)*cos(delta0) - (dY_dpsidot_fl + dY_dpsidot_fr)*sin(delta0)...
    + dX_dpsidot_rl + dX_dpsidot_rr) + v0;

A21 = (1/m_car)*((dX_du_fl + dX_du_fr)*sin(delta0) + (dY_du_fl + dY_du_fr)*cos(delta0)...
    + dY_du_rl + dY_du_rr) - psidot0 ;

A22 = (1/m_car)*((dX_dv_fl + dX_dv_fr)*sin(delta0) + (dY_dv_fl + dY_dv_fr)*cos(delta0)...
    + dY_dv_rl + dY_dv_rr) - rho_air*abs(v0)*A_side*C_air/m_car;

A23 = (1/m_car)*((dX_dpsidot_fl + dX_dpsidot_fr)*sin(delta0) + (dY_dpsidot_fl + dY_dpsidot_fr)*cos(delta0)...
    + dY_dpsidot_rl + dY_dpsidot_rr) - u0;

A31 = (1/I_z)*(l_front*(dY_du_fl + dY_du_fr)*cos(delta0) + l_front*(dX_du_fl + dX_du_fr)*sin(delta0)...
    + (l_tread/2)*(dX_du_fr - dX_du_fl)*cos(delta0) - (l_tread/2)*(dY_du_fr - dY_du_fl)*sin(delta0)...
    + (l_tread/2)*(dX_du_rr - dX_du_rl) - l_rear*(dY_du_rl + dY_du_rr));

A32 = (1/I_z)*(l_front*(dY_dv_fl + dY_dv_fr)*cos(delta0) + l_front*(dX_dv_fl + dX_dv_fr)*sin(delta0)...
    + (l_tread/2)*(dX_dv_fr - dX_dv_fl)*cos(delta0) - (l_tread/2)*(dY_dv_fr - dY_dv_fl)*sin(delta0)...
    + (l_tread/2)*(dX_dv_rr - dX_dv_rl) - l_rear*(dY_dv_rl + dY_dv_rr));

A33 = (1/I_z)*(l_front*(dY_dpsidot_fl + dY_dpsidot_fr)*cos(delta0) + l_front*(dX_dpsidot_fl + dX_dpsidot_fr)*sin(delta0)...
    + (l_tread/2)*(dX_dpsidot_fr - dX_dpsidot_fl)*cos(delta0) - (l_tread/2)*(dY_dpsidot_fr - dY_dpsidot_fl)*sin(delta0)...
    + (l_tread/2)*(dX_dpsidot_rr - dX_dpsidot_rl) - l_rear*(dY_dpsidot_rl + dY_dpsidot_rr));

% B matrix elements
B11 = (1/m_car)*((dX_ddelta_fl + dX_ddelta_fr)*cos(delta0) - (X_fl + X_fr)*sin(delta0)...
    - (dY_ddelta_fl + dY_ddelta_fr)*sin(delta0) - (Y_fl + Y_fr)*cos(delta0)...
    + dX_ddelta_rl + dX_ddelta_rr);

B12 = (1/m_car)*((dX_dXR_fl + dX_dXR_fr)*cos(delta0) - (dY_dXR_fl + dY_dXR_fr)*sin(delta0)...
    + dX_dXR_rl + dX_dXR_rr);

B21 = (1/m_car)*((dX_ddelta_fl + dX_ddelta_fr)*sin(delta0) + (X_fl + X_fr)*cos(delta0)...
    + (dY_ddelta_fl + dY_ddelta_fr)*cos(delta0) - (Y_fl + Y_fr)*sin(delta0)...
    + dY_ddelta_rl + dY_ddelta_rr);

B22 = (1/m_car)*((dX_dXR_fl + dX_dXR_fr)*sin(delta0) + (dY_dXR_fl + dY_dXR_fr)*cos(delta0)...
    + dY_dXR_rl + dY_dXR_rr);

B31 = (1/I_z)*(l_front*(dY_ddelta_fl + dY_ddelta_fr)*cos(delta0) - l_front*(Y_fl + Y_fr)*sin(delta0)...
        + l_front*(dX_ddelta_fl + dX_ddelta_fr)*sin(delta0) + l_front*(X_fl + X_fr)*cos(delta0)...
    + (l_tread/2)*(dX_ddelta_fr - dX_ddelta_fl)*cos(delta0) - (l_tread/2)*(X_fr - X_fl)*sin(delta0)...
- (l_tread/2)*(dY_ddelta_fr - dY_ddelta_fl)*sin(delta0) - (l_tread/2)*(Y_fr - Y_fl)*cos(delta0)...
    + (l_tread/2)*(dX_ddelta_rr - dX_ddelta_rl) - l_rear*(dY_ddelta_rl + dY_ddelta_rr));

B32 = (1/I_z)*(l_front*(dY_dXR_fl + dY_dXR_fr)*cos(delta0) + l_front*(dX_dXR_fl + dX_dXR_fr)*sin(delta0)...
    + (l_tread/2)*(dX_dXR_fr - dX_dXR_fl)*cos(delta0) - (l_tread/2)*(dY_dXR_fr - dY_dXR_fl)*sin(delta0)...
    + (l_tread/2)*(dX_dXR_rr - dX_dXR_rl) - l_rear*(dY_dXR_rl + dY_dXR_rr));

A = [A11 A12 A13;
     A21 A22 A23;
     A31 A32 A33];

 B = [B11 B12;
      B21 B22;
      B31 B32];
