%% Plot simulation
fig1 = figure();

% time interval between car figures
step = 0.5;

% find indices at selected time intervals
next = 0;
index = [];
for l = 1:length(simout_time)
    if simout_time(l) >= next;
       index = [index l];
       next = next+step;
    end
end

% plot car and trajecotry
plot(-simout_position(:,2), simout_position(:,1), 'r');
hold on

for now = index
    
    % rotation matrix for car
    R_car = [cos(simout_psi(now)) -sin(simout_psi(now)) ;
             sin(simout_psi(now))  cos(simout_psi(now))]';

    % rotation matrix for front wheels
    R_steering = [cos(simout_delta(now)) -sin(simout_delta(now)) ;
                  sin(simout_delta(now))  cos(simout_delta(now))]';
         
    % create car and wheels
    car         = R_car*[l_front  l_front  -l_rear  -l_rear  l_front ; l_tread/2 -l_tread/2 -l_tread/2 l_tread/2 l_tread/2];
    rear_wheel  = R_car*[l_car/10 l_car/10 -l_car/10 -l_car/10 l_car/10; l_tread/10 -l_tread/10 -l_tread/10 l_tread/10 l_tread/10];    
    front_wheel = R_steering*rear_wheel;

    % find car and wheel positions
    carpos = simout_position(now,:);        
    front_left_pos  = R_car*[l_front l_tread/2]';
    front_right_pos = R_car*[l_front -l_tread/2]';
    rear_right_pos  = R_car*[-l_rear -l_tread/2]';
    rear_left_pos  = R_car*[-l_rear l_tread/2]';

    % plot car
    plot(-carpos(2)+car(2,:), carpos(1)+car(1,:),'g')                                                           % vehicle body
    patch(-carpos(2)+front_right_pos(2)+front_wheel(2,:), carpos(1)+front_right_pos(1)+front_wheel(1,:),'r');   % front right wheel
    patch(-carpos(2)+rear_right_pos(2)+rear_wheel(2,:), carpos(1)+rear_right_pos(1)+rear_wheel(1,:),'b');       % rear right wheel
    patch(-carpos(2)+rear_left_pos(2)+rear_wheel(2,:), carpos(1)+rear_left_pos(1)+rear_wheel(1,:),'b');         % rear left wheel
    patch(-carpos(2)+front_left_pos(2)+front_wheel(2,:), carpos(1)+front_left_pos(1)+front_wheel(1,:),'r');     % front left wheel
end 

axis equal
xlabel 'East'
ylabel 'North'
title ('Simulation result', 'Interpreter','Latex','Fontsize', 14)
h = legend('Trajectory of centre of mass','Rigid body car', 'Front wheels','Rear wheels');
x = get(h, 'Position');
set(h, 'Position', x + [0.1 0.07 0 0]);

set(fig1,'Position', [600   400   700   475])

% plot waypoints
plot(-waypoints(2:4:length(waypoints)),waypoints(1:4:length(waypoints)), '-or');

%% Plot slip angles
fig2 = figure(2);
    hold on
    grid on
    plot(simout_time, simout_alpha(:,1).*180/pi, 'r');
    plot(simout_time, simout_alpha(:,2).*180/pi, 'm');
    plot(simout_time, simout_alpha(:,3).*180/pi, 'c');
    plot(simout_time, simout_alpha(:,4).*180/pi, 'b');    
    xlabel 'Time [s]'
    ylabel 'Angle [deg]'
    title ('Wheel slip angles', 'Interpreter','Latex','Fontsize', 14)
    h = legend('\alpha_{FL}','\alpha_{FR}','\alpha_{RL}','\alpha_{RR}');
    x = get(h, 'Position');
    set(h, 'Position', x + [0.05 0.07 0 0]);

set(fig2,'Position', [600   400   800   475])

%% Plot acceleration and normal force
fig3 = figure(3);
subplot(2,1,1)
    hold on
    grid on
    plot(simout_time, simout_Fz_wheel(:,1), 'r');
    plot(simout_time, simout_Fz_wheel(:,2), 'm');
    plot(simout_time, simout_Fz_wheel(:,3), 'c');
    plot(simout_time, simout_Fz_wheel(:,4), 'b');    
    xlabel 'Time [s]'
    ylabel 'Force [N]'
    title ('Normal force acting on each wheel', 'Interpreter','Latex','Fontsize', 14)
    h = legend('F_{z, FL}','F_{z, FR}','F_{z, RL}','F_{z, RR}');
    x = get(h, 'Position');
    set(h, 'Position', x + [0.05 0.07 0 0]);

subplot(2,1,2)
    hold on
    grid on
    plot(simout_time, simout_acc(:,1) ./ 9.81, 'r');
    plot(simout_time, simout_acc(:,2) ./ 9.81, 'g');    
    xlabel 'Time [s]'
    ylabel 'Acceleration [g]'
    title ('Vehicle acceleration', 'Interpreter','Latex','Fontsize', 14)
    h = legend('Longitudinal acceleration','Lateral acceleration');
    x = get(h, 'Position');
    set(h, 'Position', x + [0.05 0.07 0 0]);
    
set(fig3,'Position', [600   400   800   475]) 

%% Plot longitudinal and lateral slip
fig4 = figure(4);
subplot(2,1,1)
    hold on
    grid on
    plot(simout_time, simout_sigma_long(:,1), 'r');
    plot(simout_time, simout_sigma_long(:,2), 'm');
    plot(simout_time, simout_sigma_long(:,3), 'c');
    plot(simout_time, simout_sigma_long(:,4), 'b');    
    xlabel 'Time [s]'
    ylabel 'Slip'
    title ('Longitudinal wheel slip', 'Interpreter','Latex','Fontsize', 14)
    h = legend('s_{L,FL}','s_{L,FR}','s_{L,RL}','s_{L,RR}');
    x = get(h, 'Position');
    set(h, 'Position', x + [0.05 0.07 0 0]);

subplot(2,1,2)
    hold on
    grid on
    plot(simout_time, simout_sigma_lat(:,1), 'r');
    plot(simout_time, simout_sigma_lat(:,2), 'm');
    plot(simout_time, simout_sigma_lat(:,3), 'c');
    plot(simout_time, simout_sigma_lat(:,4), 'b');    
    xlabel 'Time [s]'
    ylabel 'Slip'
    title ('Lateral wheel slip', 'Interpreter','Latex','Fontsize', 14)
    h = legend('s_{S,FL}','s_{S,FR}','s_{S,RL}','s_{S,RR}');
    x = get(h, 'Position');
    set(h, 'Position', x + [0.05 0.07 0 0]);
    
set(fig4,'Position', [600   400   800   475])

%% Plot velocity and actuator input
fig5 = figure(5);
subplot(2,1,1)
    hold on
    grid on
    plot(simout_time, simout_delta(:,1) .* 180/pi, 'g');
    plot(simout_time, simout_torque(:,1), 'r');        
    xlabel 'Time [s]'
    ylabel 'Angle [deg], Torque [Nm]'
    title ('Steering angle and motor torque', 'Interpreter','Latex','Fontsize', 14)
    h = legend('Steering angle \delta', 'Motor torque');
    x = get(h, 'Position');
    set(h, 'Position', x + [0.05 0.07 0 0]);
    
subplot(2,1,2)
    hold on
    grid on
    plot(simout_time, simout_vbody(:,1), 'r');
    plot(simout_time, simout_vbody(:,2), 'g');    
    xlabel 'Time [s]'
    ylabel 'Velocity [m/s]'
    title ('Vehicle velocities', 'Interpreter','Latex','Fontsize', 14)
    h = legend('u','v');
    x = get(h, 'Position');
    set(h, 'Position', x + [0.05 0.07 0 0]);
    
set(fig5,'Position', [600   400   800   475])    

% Plot wheel speeds
fig6 = figure(6);
hold on
grid on
plot(simout_time, simout_omega_wheel(:,1), 'g');
plot(simout_time, simout_omega_wheel(:,2), 'm');
plot(simout_time, simout_omega_wheel(:,3), 'b');
plot(simout_time, simout_omega_wheel(:,4), 'r');

xlabel 'Time [s]'
ylabel 'Wheel angular velocity [rad/s]'
title ('Wheel angular veolcity', 'Interpreter','Latex','Fontsize', 14)
h = legend('Front left', 'Front right', 'Rear left', 'Rear right');
x = get(h, 'Position');
set(h, 'Position', x + [0.05 0.07 0 0]);
    
set(fig6,'Position', [600   400   800   475])    

%% Plot motor torque
fig7 = figure();

subplot(2,1,1)
hold on
plot(simout_time, simout_torque);
grid on
xlabel 'Time [s]'
ylabel 'Torque [Nm]'
title ('Motor Torque', 'Interpreter','Latex','Fontsize', 14)
legend 'Motor Torque'

subplot(2,1,2)

[AX, H1, H2] = plotyy(simout_time, sqrt(simout_vbody(:,1).^2+simout_vbody(:,2).^2), simout_time, simout_throttle);
grid on
set(get(AX(1),'Ylabel'),'String','Speed [m/s]') 
set(get(AX(2),'Ylabel'),'String','Value [0-254]') 

xlabel 'Time [s]'
title ('Throttle and Speed', 'Interpreter','Latex','Fontsize', 14)
h = legend('Vehicle speed', 'Throttle');
x = get(h, 'Position');
set(h, 'Position', x + [0.05 0.07 0 0]);
set(fig7,'Position', [600   400   800   475]) 

%% Plot control input forces and moment
fig8 = figure(8);
subplot(3,1,1)
plot(simout_time,tau(:,1),'b');
xlabel('Time [s]')
ylabel('Longitudinal controller force [N]')
grid on
title('Controller input in X-direction','Interpreter','Latex','Fontsize', 14)
h = legend('X_w');
x = get(h, 'Position');
set(h, 'Position', x + [0.05 0.07 0 0]);
set(fig8,'Position', [600   400   800   475]) 

subplot(3,1,2)
plot(simout_time,tau(:,2),'b');
xlabel('Time [s]')
ylabel('Lateral controller force [N]')
grid on
title('Controller input in Y-direction','Interpreter','Latex','Fontsize', 14)
h = legend('Y_w');
x = get(h, 'Position');
set(h, 'Position', x + [0.05 0.07 0 0]);
set(fig8,'Position', [600   400   800   475]) 

subplot(3,1,3)
plot(simout_time,tau(:,3),'b');
xlabel('Time [s]')
ylabel('Yaw controller moment [Nm]')
grid on
title('Controller input about Z-axis','Interpreter','Latex','Fontsize', 14)
h = legend('N_w');
x = get(h, 'Position');
set(h, 'Position', x + [0.05 0.07 0 0]);
set(fig8,'Position', [600   400   800   475]) 

%% Plot desired vs actual speed, sideslip angle and yaw rate
fig9 = figure(9);

subplot(3,1,1)
plot(simout_time,speed,'b',simout_time,V0*ones(length(simout_time),1),'m');
xlabel('Time [s]')
ylabel('Speed [m/s]')
grid on
title('Vehicle Speed','Interpreter','Latex','Fontsize', 14)
h = legend({'Sideslip angle $\beta$','Desired sideslip angle $\beta_d$'}, 'Interpreter', 'Latex');
x = get(h, 'Position');
set(h, 'Position', x + [0.05 0.07 0 0]);
set(fig9,'Position', [600   400   800   475])

subplot(3,1,2)
plot(simout_time,simout_beta*180/pi,'b',simout_time,beta_dsat*180/pi,'m');
xlabel('Time [s]')
ylabel('Sideslip angle [deg]')
grid on
title('Sideslip angle','Interpreter','Latex','Fontsize', 14)
h = legend({'Sideslip angle $\beta$','Desired sideslip angle $\beta_d$'}, 'Interpreter', 'Latex');
x = get(h, 'Position');
set(h, 'Position', x + [0.05 0.07 0 0]);
set(fig9,'Position', [600   400   800   475])

subplot(3,1,3)
plot(simout_time,simout_psidot*180/pi,'b',simout_time,simout_controller(:,3)*180/pi,'m');
xlabel('Time [s]')
ylabel('Yaw rate [deg/s]')
grid on
title('Yaw rate','Interpreter','Latex','Fontsize', 14)
h = legend({'Yaw rate $\dot{\psi}$','$\alpha$'}, 'Interpreter', 'Latex');
x = get(h, 'Position');
set(h, 'Position', x + [0.05 0.07 0 0]);
set(fig9,'Position', [600   400   800   475])


%% Plot error states
fig10 = figure(10);
subplot(3,1,1)
plot(simout_time,simout_controller(:,1),'b')
xlabel('Time [s]')
ylabel('Error state 1 [deg]')
grid on
title('$z_1$','Interpreter','Latex','Fontsize', 14)
h = legend({'$z_1$'}, 'Interpreter', 'Latex');
x = get(h, 'Position');
set(h, 'Position', x + [0.05 0.07 0 0]);
set(fig10,'Position', [600   400   800   475])

subplot(3,1,2)
plot(simout_time,simout_controller(:,1),'b')
xlabel('Time [s]')
ylabel('Error state 2 [deg/s]')
grid on
title('$z_2$','Interpreter','Latex','Fontsize', 14)
h = legend({'$z_2$'}, 'Interpreter', 'Latex');
x = get(h, 'Position');
set(h, 'Position', x + [0.05 0.07 0 0]);
set(fig10,'Position', [600   400   800   475])

subplot(3,1,3)
plot(simout_time,thetahat)
xlabel('Time [s]')
ylabel('error estimate [deg/s]')
grid on
title('thetahat','Interpreter','Latex','Fontsize', 14)
h = legend({'$\theta$'}, 'Interpreter', 'Latex');
x = get(h, 'Position');
set(h, 'Position', x + [0.05 0.07 0 0]);
set(fig10,'Position', [600   400   800   475])

