function [nu_body_dot, a_body] = rigid_body_motion(nu_body, tau, m_car, I_car)

v_body = nu_body(1:3);
omega_body = nu_body(4:6);

% moments of inertia
I_g = diag(I_car);

% mass matrix
M_RB = [m_car*eye(3) zeros(3,3);
        zeros(3,3)   diag(I_car)];

% coriolis and centripetal matrix
C_RB = [m_car*Smtrx(omega_body) zeros(3,3);
        zeros(3,3)  -Smtrx(I_g*omega_body)];

% damping matrix
D_RB = zeros(6,6);

% gravitational vector
g = [0 0 -m_car*9.81 0 0 0]';

% translatory and rotational acceleration
nu_body_dot = M_RB\(tau - C_RB*nu_body - D_RB*nu_body - g);

% total acceleration
a_body = nu_body_dot;

% ignore heave, roll, pitch
nu_body_dot(3:5) = 0;


