% vehicle constants
l_front = 0.30;                 % length from CG to front axle [m]
l_rear  = 0.26;                 % length from CG to rear axle [m]
l_tread = 0.38;                 % wheel tread [m]
l_car = l_front + l_rear;                   % length of car [m]
A_front = 0.0285;               % vehicle front and side area [m^2]
A_side = 0.0420;                % vehicle front and side area [m^2]
m_car = 7.0;                    % vehicle mass [kg]
I_z = 0.4535;                   % moment of inertia around vehicle z-axis [kg*m^2]
C_air = 1.07;                   % air resistance coefficient
C_rolling = 0.01;               % rolling resistance coefficient
rho_air = 1.2928;               % air density (kg/m^3)
g = 9.81;                       % gravity acceleration (m/s^2)
road_condition = 5;             % wet cobblestone chosen
r_wheel = 0.085;                % wheel radius [m] (Tarmac Buster)

% distances between CG and each wheel
r_fl = sqrt(l_front^2 + (l_tread/2)^2);
r_fr = r_fl;
r_rl = sqrt(l_rear^2 + (l_tread/2)^2);
r_rr = r_rl;

% angles from CG to each wheel
theta_fl = atan2(l_tread/2, l_front);
theta_fr = atan2(l_front, l_tread/2);
theta_rl = atan2(l_rear, l_tread/2);
theta_rr = atan2(l_tread/2, l_rear);

V0values = 1:10;
beta0values = -(1:60)*pi/180;
step = 1e-8;
eigMatrix = zeros(3,10,30);
A_matrices = 0;
B_matrices = 0;



for i = 1:30
    for j = 1:10
        if passedss(i,j) == 1
% finds the derivatives of the friction coefficients
[dmylong_du_fl,dmylong_du_fr,dmylong_du_rl,dmylong_du_rr, dmylong_dv_fl,dmylong_dv_fr,dmylong_dv_rl,dmylong_dv_rr,...
dmylong_dpsidot_fl,dmylong_dpsidot_fr,dmylong_dpsidot_rl,dmylong_dpsidot_rr,dmylong_ddelta_fl,dmylong_ddelta_fr,dmylong_ddelta_rl,dmylong_ddelta_rr,...
dmylong_dXR_fl,dmylong_dXR_fr,dmylong_dXR_rl,dmylong_dXR_rr,dmylat_du_fl,dmylat_du_fr,dmylat_du_rl,dmylat_du_rr,...
dmylat_dv_fl,dmylat_dv_fr,dmylat_dv_rl,dmylat_dv_rr, dmylat_dpsidot_fl,dmylat_dpsidot_fr,dmylat_dpsidot_rl,dmylat_dpsidot_rr,...
dmylat_ddelta_fl,dmylat_ddelta_fr,dmylat_ddelta_rl,dmylat_ddelta_rr,dmylat_dXR_fl,dmylat_dXR_fr,dmylat_dXR_rl,dmylat_dXR_rr,mylong,mylat] = find_mu(road_condition,...
deltass, omega_mss,omega_flss, step, V0values, beta0values,l_front,l_rear,l_tread,r_wheel,r_fl,r_fr,theta_fl,theta_fr);

beta0 = beta0values(i);
V0 = V0values(j);

delta0 = deltass(i,j);
omega_m0 = omega_mss(i,j);
omega_fl0 = omega_flss(i,j);

Vrot_fl = omega_fl0*r_wheel;
r0 = ((V0*(cos(beta0)*cos(delta0) + sin(beta0)*sin(delta0)) - Vrot_fl)/(r_fl*(sin(theta_fl)*cos(delta0) - cos(theta_fl)*sin(delta0))));
v_wheel_fr0 = [V0*cos(beta0) + r0*r_fr*cos(theta_fr); V0*sin(beta0) + r0*r_fr*sin(theta_fr)];
Vw_fr0 = norm(v_wheel_fr0);
omega_fr0 = (1/r_wheel)*(cos(delta0 - atan2(v_wheel_fr0(2),v_wheel_fr0(1)))*Vw_fr0);
omega_wheel = [omega_fl0;omega_fr0;omega_m0;omega_m0];

% runs linearisation
[A,B] = linearize(V0,beta0,r0,delta0,omega_m0,omega_fl0,dmylong_du_fl(i,j),dmylong_du_fr(i,j),dmylong_du_rl(i,j),dmylong_du_rr(i,j), dmylong_dv_fl(i,j),dmylong_dv_fr(i,j),...
    dmylong_dv_rl(i,j),dmylong_dv_rr(i,j),dmylong_dpsidot_fl(i,j),dmylong_dpsidot_fr(i,j),dmylong_dpsidot_rl(i,j),dmylong_dpsidot_rr(i,j),dmylat_du_fl(i,j),...
    dmylat_du_fr(i,j),dmylat_du_rl(i,j),dmylat_du_rr(i,j),dmylat_dv_fl(i,j),dmylat_dv_fr(i,j),dmylat_dv_rl(i,j),dmylat_dv_rr(i,j), dmylat_dpsidot_fl(i,j),...
    dmylat_dpsidot_fr(i,j),dmylat_dpsidot_rl(i,j),dmylat_dpsidot_rr(i,j),dmylong_ddelta_fl(i,j),dmylong_ddelta_fr(i,j),dmylong_ddelta_rl(i,j),dmylong_ddelta_rr(i,j),...
    dmylat_ddelta_fl(i,j),dmylat_ddelta_fr(i,j),dmylat_ddelta_rl(i,j),dmylat_ddelta_rr(i,j),dmylong_dXR_fl(i,j),dmylong_dXR_fr(i,j),dmylong_dXR_rl(i,j),dmylong_dXR_rr(i,j),...
    dmylat_dXR_fl(i,j),dmylat_dXR_fr(i,j),dmylat_dXR_rl(i,j),dmylat_dXR_rr(i,j),mylong,mylat,l_front,l_rear,l_tread,l_car,A_front,A_side,m_car,I_z,C_air,C_rolling,rho_air,g,...
    r_fl,r_fr,r_rl,r_rr,theta_fl,theta_fr,theta_rl,theta_rr);

                % finds linearised system A and B matrices
                if j == 3 
                    if A_matrices == 0
                        A_matrices = [A(1,:);
                                      A(2,:);
                                      A(3,:)];
                        B_matrices = [B(1,:);
                                      B(2,:);
                                      B(3,:)];
                    else
                        A_matrices = [A_matrices;
                                      A(1,:);
                                      A(2,:);
                                      A(3,:)];
                        B_matrices = [B_matrices;
                                      B(1,:);
                                      B(2,:);
                                      B(3,:)];
                    end  
                end
                
            % extract eigenvalues
            eigMatrix(:,j,i) = eig(A);
        end
    end
end

% location of poles
P = [-2;-5;-5];

% gain matrix for PID controller
K = place(A,B,P)
