% finds the steady state values by a sweep in rear wheel rotation speed, 
% front left wheel rotation speed and steering angle

n = 30; % maximum sideslip angle
m = 10; % maximum speed
search = 1; % start a search for steady state values
initvalues = 0; % initalise all steady state values to zero
improve = 1; % improve the initial values found

a1 = 3; % relationship between produced force and throttle input
a2 = 6; % relationship between produced force and steering slip angle
l_front = 0.30;                 % length from CG to front axle [m]
l_rear  = 0.26;                 % length from CG to rear axle [m]
l_tread = 0.38;                 % wheel tread [m]
m_car = 7.0;                    % vehicle mass [kg]
r_wheel = 0.085;                % wheel radius [m] (Tarmac Buster)
road_condition = 5;             % road condition, 1=dry asphalt, 2=wet asphalt, 3=dry concrete
                                %                 4=dry cobblestones, 5=wet cobblestones 6=snow, 7=ice      
l_car = l_front + l_rear;                   % length of car [m]
C_rolling = 0.01;               % rolling resistance coefficient
C_air = 1.07;                   % air resistance coefficient
rho_air = 1.2928;                           % air density (kg/m^3)
A_front = 0.0285;               % vehicle front and side area [m^2]
A_side = 0.0420;                % vehicle front and side area [m^2]
h_cg = 0.075;                   % height of centre of gravity [m]
I_x = (m_car/12)*(h_cg^2 + l_tread^2);      % moment of inertia around vehicle x-axis [kg*m^2]
I_y = (m_car/12)*(l_car^2 + h_cg^2);        % moment of inertia around vehicle y-axis [kg*m^2]
I_z = 0.4535;                   % moment of inertia around vehicle z-axis [kg*m^2]
I_car = [I_x I_y I_z];                      % vector of vehicle moments of inertia

%     % dc motor constants
R_a = 0.8;                      % armature resistance [ohm]
k_E = 0.17;                     % back EMF constant [V*s/rad]
k_t = 0.17;                     % torque constant [Nm/A]

r_fl = sqrt(l_front^2 + (l_tread/2)^2); % length from cg to front left wheel [m]
r_fr = r_fl;                            % length from cg to front right wheel [m]
theta_fl = atan2(l_tread/2, l_front);   % angle between cg to front left wheel [rad]
theta_fr = atan2(l_front, l_tread/2);   % angle between cg to front left right [rad]

% the search is started
if search == 1
    if initvalues == 2
        passedss = zeros(n,m); % upholds constraint(1) or not(0)
        deltass = zeros(n,m);   % steering angle [rad/s]
        omega_mss = zeros(n,m); % rear wheel angular velocity [rad/s]
        omega_flss = zeros(n,m); % front left wheel angular velocity [rad/s]
        omega_frss = zeros(n,m); % front right wheel angular velocity [rad/s]
        valuess = zeros(n,m); % acceleration value at steady state [m/s^2]
        X_Rinputss = zeros(n,m); % thrust force [N]
        psi_dotss = zeros(n,m); % yaw rate [rad/s]
        u_a0ss = zeros(n,m); % throttle value [V]
        i_a0ss = zeros(n,m); % motor armature current [A]
    end
    for i = 1:n
        for j = 1:m
    V0 = j;
    beta0 = -i*pi/180;
    stepsize = 0;
    
    % finds all the steady state values
    [passed,deltabest,omega_mbest,omega_flbest,omega_frbest,...
        totbestcheck,u_a0best,i_a0best,X_R0input,r0best] =...
        find_steadystate(V0,beta0,deltass(i,j),omega_mss(i,j),omega_flss(i,j),valuess(i,j),stepsize,l_front,l_rear,l_tread,m_car,r_wheel,road_condition,C_rolling,C_air,rho_air,A_front,A_side,...
        h_cg,I_car,a1,a2,R_a,k_E,k_t,r_fl,r_fr,theta_fl,theta_fr,l_car,0);
                valuess(i,j) = totbestcheck;
                passedss(i,j)= passed;
                deltass(i,j) = deltabest;
                omega_mss(i,j) = omega_mbest;
                omega_flss(i,j) = omega_flbest;
                omega_frss(i,j) = omega_frbest;
                i_a0ss(i,j) = i_a0best;
                u_a0ss(i,j) = u_a0best;
                X_Rinputss(i,j) = X_R0input;
                psi_dotss(i,j) = r0best;
        end
    end
else
    load('steady_state_sim')
end

% improvements are made on initial findings
if improve == 1
    for i = 1:n
        for j = 1:m
             for k = 1:5
                beta0 = -i*pi/180;
                V0 = j;
                stepsize = 10^(-k);
        if passedss(i,j) == 1
                [passed,deltabest,omega_mbest,omega_flbest,omega_frbest,...
        totbestcheck,u_a0best,i_a0best,X_R0input,r0best] =...
                     find_steadystate(V0,beta0,deltass(i,j),omega_mss(i,j),...
                     omega_flss(i,j),valuess(i,j),stepsize,l_front,l_rear,l_tread,m_car,r_wheel,road_condition,C_rolling,C_air,rho_air,A_front,A_side,...
        h_cg,I_car,a1,a2,R_a,k_E,k_t,r_fl,r_fr,theta_fl,theta_fr,l_car,improve);
            if totbestcheck < valuess(i,j)
                    valuess(i,j) = totbestcheck;
                    passedss(i,j)= passed;
                    deltass(i,j) = deltabest;
                    omega_mss(i,j) = omega_mbest;
                    omega_flss(i,j) = omega_flbest;
                    omega_frss(i,j) = omega_frbest;
                    i_a0ss(i,j) = i_a0best;
                    u_a0ss(i,j) = u_a0best;
                    X_Rinputss(i,j) = X_R0input;
                    psi_dotss(i,j) = r0best;
            end
        end
             end 
        end
    end
end
